#!/usr/bin/python

# Verify system call parameters

import tempfile
import subprocess
import time
import os
import signal
import re
import sys
import atexit

def cleanup():
	subprocess.check_call(['auditctl', '-D'], stdout=devnull)
	os.kill(auditd.pid, signal.SIGKILL)

atexit.register(cleanup)

devnull = open('/dev/null', 'w')
subprocess.check_call(['auditctl', '-D'], stdout=devnull)

time.sleep(0.1)

subprocess.check_call(['auditctl', '-a', 'exit,always', '-F', 'arch=b64', '-S', 'sched_yield'])

time.sleep(0.1)

allargs = [
	[ 0, 0, 0, 0],
	[ 1, 2, 3, 4],
	[ 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff],
	[ 0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff],
]

r = re.compile("type=SYSCALL.*a0=([0-9A-Fa-f]+).*a1=([0-9A-Fa-f]+).*a2=([0-9A-Fa-f]+).*a3=([0-9A-Fa-f]+)")

for args in allargs:
	logfile = tempfile.TemporaryFile()
	auditd = subprocess.Popen(['auditd', '-f', '-s', 'enable'],
				  stdout=logfile, stderr=devnull)

	time.sleep(1)

	cmd = [ './sched_yield', '%x' % args[0],  '%x' % args[1],
		'%x' % args[2], '%x' % args[3]]
	subprocess.check_call(cmd)

	time.sleep(1)

	logfile.seek(0)
	found = False

	for line in logfile.readlines():
		m = r.match(line)
		if m:
			newargs = [ int(m.group(1), 16), int(m.group(2), 16),
				    int(m.group(3), 16), int(m.group(4), 16) ]

			if args != newargs:
				print "Args don't match:"
				print "expected:\t ",
				print args
				print "actual:\t ",
				print newargs
				exit(1)

			found = True

	if found == False:
		print "Could not find syscall audit entry for ",
		print args
		print 'FAIL'
		sys.exit(1)

	logfile.close()
	os.kill(auditd.pid, signal.SIGKILL)
