#!/usr/bin/python

# Make sure syscall auditing rules added before auditd is started are obeyed

import tempfile
import subprocess
import time
import os
import signal
import re
import sys
import atexit

def cleanup():
	subprocess.check_call(['auditctl', '-D'], stdout=devnull)
	os.kill(auditd.pid, signal.SIGKILL)

atexit.register(cleanup)

devnull = open('/dev/null', 'w')
subprocess.check_call(['auditctl', '-D'], stdout=devnull)

time.sleep(0.1)

# Keep a handle so we don't delete temp file until we exit
f = tempfile.NamedTemporaryFile()
filename = f.name

subprocess.check_call(['auditctl', '-a', 'exit,always', '-F', 'arch=b64', '-S', 'open', '-F' 'path=%s' % filename])

time.sleep(0.1)

logfile = tempfile.TemporaryFile()
auditd = subprocess.Popen(['auditd', '-f', '-s', 'enable'], stdout=logfile, stderr=devnull)

time.sleep(1)

subprocess.check_call(['./open_once.py', filename])

time.sleep(0.1)

expected = [
	'type=SYSCALL.*comm="open_once.py"',
	'type=CWD',
	'type=PATH.*name="%s"' % filename
]

for expect in expected:
	r = re.compile(expect)
	logfile.seek(0)
	found = False
	for line in logfile.readlines():
		if r.match(line):
			found = True

	if found == False:
		print 'Could not find audit entry containing %s' % expect
		print 'FAIL'
		sys.exit(1)
