#!/usr/bin/python

# Verify 32 and 64bit arch parameter in syscall audit logs

import tempfile
import subprocess
import time
import os
import signal
import re
import sys
import atexit

def cleanup():
	subprocess.check_call(['auditctl', '-D'], stdout=devnull)
	os.kill(auditd.pid, signal.SIGKILL)

atexit.register(cleanup)

uname = subprocess.check_output(['uname', '-m'])
if 'ppc64' in uname:
	arch_64bit = 0x80000015
	arch_32bit = 0x14
elif 'x86_64' in uname:
	arch_64bit = 0xc000003e
	arch_32bit = 0x40000003

devnull = open('/dev/null', 'w')
subprocess.check_call(['auditctl', '-D'], stdout=devnull)

time.sleep(0.1)

subprocess.check_call(['auditctl', '-a', 'exit,always', '-F', 'arch=b64', '-S', 'sched_yield'])
subprocess.check_call(['auditctl', '-a', 'exit,always', '-F', 'arch=b32', '-S', 'sched_yield'])

time.sleep(0.1)

r = re.compile("type=SYSCALL.*arch=([0-9A-Fa-f]+)")


tests = [
	[ './sched_yield', arch_64bit ],
	[ './sched_yield_32', arch_32bit ]
]

for test in tests:
	logfile = tempfile.TemporaryFile()
	auditd = subprocess.Popen(['auditd', '-f', '-s', 'enable'],
				  stdout=logfile, stderr=devnull)

	time.sleep(1)

	cmd = [ test[0], '0', '0', '0', '0' ]
	subprocess.check_call(cmd)

	time.sleep(1)

	logfile.seek(0)
	found = False

	for line in logfile.readlines():
		m = r.match(line)
		if m:
			arch = int(m.group(1), 16)
			if arch != test[1]:
				print 'Incorrect arch: got 0x%x expected 0x%x' % (arch, test[1])
				exit(1)

			found = True

	if found == False:
		print "Could not find syscall audit entry for %s " % test[0]
		print 'FAIL'
		sys.exit(1)

	logfile.close()
	os.kill(auditd.pid, signal.SIGKILL)
